/* Sistemas Integrados TEKNISA

Nome: 0001_MAKSTPO_ESTRUTURA_TAB.SQL

@C:\web_580\sql\end_conv\oracle\clientes\mak\0001_MAKSTRO_ESTRUTURA_TAB.SQL

Descrio:
Alterao da estrutura das tabelas dos sistema TEKNISA para atender a integrao
MAKRO

Criao:
29/06/2012 - Fabrcio Marcelo da Silva
*/


-- ============================================================
--   TABLE: PRODUTO
-- ============================================================
ALTER TABLE PRODUTO ADD USR_DTEXPPROD DATE NULL
/

-- ============================================================
--   TABLE: LANCTOESTOQ
-- ============================================================
ALTER TABLE LANCTOESTOQ ADD USR_CHAVEITENS VARCHAR2(09) NULL
/

-- ============================================================
--   Index: USR_CHAVE_MAKRO
-- ============================================================
create index USR_CHAVE_MAKRO on LANCTOESTOQ (USR_CHAVEITENS asc)
/

-- ============================================================
--   Table: USR_PRODMAKRO
-- ============================================================
create table USR_PRODMAKRO
(
    CDPRODMAKRO         VARCHAR2(06)           not null,
    CDPRODUTO           VARCHAR2(10)           not null,
    NMPRODMAKRO         VARCHAR2(33)           not null,
    constraint PK_USR_PRODMAKRO primary key (CDPRODMAKRO)
)
/

-- ============================================================
--   Table: USR_CABEC_REQUI
-- ============================================================
create table USR_CABEC_REQUI
(
    CDFILIAL            VARCHAR2(03)           not null,
    NRREQUISICAO        VARCHAR2(06)           not null,
    DTENTRADA           DATE                   not null,
    DTIMPORTACAO        DATE                   null,
    constraint PK_USR_CABEC_REQUI primary key (CDFILIAL, NRREQUISICAO)
)
/

-- ============================================================
--   Table: USR_ITENS_REQUI
-- ============================================================
create table USR_ITENS_REQUI
(
    CDFILIAL            VARCHAR2(03)           not null,
    NRREQUISICAO        VARCHAR2(06)           not null,
    CDPRODMAKRO         VARCHAR2(06)           not null,
    VRQUANTIDADE        NUMBER(10,3)           not null,
    VRVALRUNIT          NUMBER(12,2)           not null,
    constraint PK_USR_ITENS_REQUI primary key (CDFILIAL, NRREQUISICAO, CDPRODMAKRO)
)
/

alter table USR_ITENS_REQUI
    add constraint USR_FK_USR_CABEC_REQUI foreign key  (CDFILIAL, NRREQUISICAO)
       references USR_CABEC_REQUI (CDFILIAL, NRREQUISICAO) on delete cascade
/


-- ============================================================
--   Table: USR_CABEC_VENDA
-- ============================================================
create table USR_CABEC_VENDA
(
    CDFILIAL            VARCHAR2(03)           not null,
    CDLOJA              VARCHAR2(10)           not null,
    CDCAIXA             VARCHAR2(03)           not null,
    NRSEQVENDA          VARCHAR2(10)           not null,
    DTVENDA             DATE                   not null,
    VRTOTALVENDA        NUMBER(12,2)           not null,
    VRTOTALDESCONTO     NUMBER(12,2)           not null,
    DSOBSERVERROS       VARCHAR2(200)          null,
    DTIMPORTACAO        DATE                   null,
    constraint PK_USR_CABEC_VENDA primary key (CDFILIAL, CDCAIXA, NRSEQVENDA)
)
/

-- ============================================================
--   Table: USR_ITENS_VENDA
-- ============================================================
create table USR_ITENS_VENDA
(
    CDFILIAL            VARCHAR2(03)           not null,
    CDLOJA              VARCHAR2(10)           not null,
    CDCAIXA             VARCHAR2(03)           not null,
    NRSEQVENDA          VARCHAR2(10)           not null,
    CDPRODUTO           VARCHAR2(20)           not null,
    VRQUANTIDADE        NUMBER(10,3)           not null,
    VRVALRUNIT          NUMBER(12,2)           not null,
    VRDESCONTO          NUMBER(12,2)           not null,
    constraint PK_USR_ITENS_VENDA primary key (CDFILIAL, CDCAIXA, NRSEQVENDA, CDPRODUTO)
)
/

alter table USR_ITENS_VENDA
    add constraint USR_FK_USR_CABEC_VENDA foreign key  (CDFILIAL, CDCAIXA, NRSEQVENDA)
       references USR_CABEC_VENDA (CDFILIAL, CDCAIXA, NRSEQVENDA) on delete cascade
/

-- ============================================================
--   Index: USR_INDEX_VENDA
-- ============================================================
create index USR_INDEX_VENDA on USR_CABEC_VENDA (DTVENDA asc, DTIMPORTACAO asc)
/

-- ============================================================
--   Index: USR_INDEX_VENDA_02
-- ============================================================
create index USR_INDEX_VENDA_02 on USR_CABEC_VENDA (CDFILIAL asc, DTVENDA asc, DTIMPORTACAO asc)
/

-- ============================================================
--   Table: USR_REDUCAOZ
-- ============================================================
create table USR_REDUCAOZ
(
    CDFILIAL            VARCHAR2(03)           not null,
    CDCAIXA             VARCHAR2(03)           not null,
    NRREDUCAOZ          VARCHAR2(10)           not null,
    DTREDUCAOZ          DATE                   not null,
    REINICIO            VARCHAR2(10)           null,
    SERIEREDZ           VARCHAR2(30)           null,
    CONTREDZ            VARCHAR2(10)           null,
    LEITURAX            VARCHAR2(10)           null,
    COMPNFISCAL         VARCHAR2(10)           null,
    CANCCUPFISC         VARCHAR2(10)           null,
    CUPFISCINI          VARCHAR2(10)           null,
    CUPFISCFIN          VARCHAR2(10)           null,
    VRTOTALINI          NUMBER(12,2)           null,
    VRTOTALFIN          NUMBER(12,2)           null,
    VRVENDABRUT         NUMBER(12,2)           null,
    VRTOTALISS          NUMBER(12,2)           null,
    VRTOTALCANC         NUMBER(12,2)           null,
    VRTOTALDESC         NUMBER(12,2)           null,
    VRVENDALIQU         NUMBER(12,2)           null,
    VRTOTALACRE         NUMBER(12,2)           null,
    ORIGEMREDU          VARCHAR2(03)           null,
    DSOBSERVERROS       VARCHAR2(200)          null,
    DTIMPORTACAO        DATE                   null,
    constraint PK_USR_REDUCAOZ primary key (CDFILIAL, CDCAIXA, NRREDUCAOZ, DTREDUCAOZ)
)
/

-- ============================================================
--   Index: USR_INDEX_REDUCAOZ
-- ============================================================
create index USR_INDEX_REDUCAOZ on USR_REDUCAOZ (DTREDUCAOZ asc, DTIMPORTACAO asc)
/

-- ============================================================
--   Index: USR_INDEX_REDUCAOZ_02
-- ============================================================
create index USR_INDEX_REDUCAOZ_02 on USR_REDUCAOZ (CDFILIAL asc, DTREDUCAOZ asc, DTIMPORTACAO asc)
/

-- ============================================================
--   Index: USR_CHAVE_MAKRO
-- ============================================================
drop index USR_CHAVE_MAKRO
/

create index USR_CHAVE_MAKRO on LANCTOESTOQ (USR_CHAVEITENS, CDFILIAL, DTLANCESTQ asc)
/


-- VERSAO --
BEGIN ATUVEROBJ('ATUALIZACAO_ESTRUTURA_BD','5.80.XXX'); END;
/


